local L = LibStub("AceLocale-3.0"):GetLocale("ShinyStats")
if not L then return end

ShinyStats = LibStub("AceAddon-3.0"):NewAddon("ShinyStats")
local stats,enchColor,shortenEnchants, shortenEquips = nil, nil, nil, nil

local defaults = {
	profile = {
		shortenEnchants = true,
		shortenEquips	= true,
		colors = {
			["agi"] = "ffee03", 	-- Agility
			["sta"] = "dda357", 	-- Stamina
			["spi"] = "b7ffb8", 	-- Spirit
			["int"] = "b395ff", 	-- Intellect
			["str"] = "ff000a", 	-- Strength
			
			["exp"] = "bbff00", 	-- Expertise
			["cri"] = "e8850d", 	-- Crit Rating
			["hit"] = "00ffc7", 	-- Hit Rating
			["has"] = "b3eeff", 	-- Haste Rating
			["ap"] = "ff494f", 		-- Attack Power
			
			["frr"] = "188bff", 	-- Frost Res
			["fr"] = "ff8300", 		-- Fire Res
			["nr"] = "87ff3c", 		-- Nature Res
			["sr"] = "bd33b1", 		-- Shadow Res
			["ar"] = "a0f2ff", 		-- Arcane Res
			["hr"] = "f3b8ff", 		-- Holy Res
			
			["ilvl"] = "F2C038", 	-- Item Level
			
			["def"] = "3f63ff", 	-- Defense
			["blo"] = "a2793c", 	-- Block
			["par"] = "f600ff", 	-- Parry
			["dod"] = "bcff00", 	-- Dodge
			["res"] = "686eff",		-- Resilience
			
			["dmg"] = "ff8930", 	-- Damage
			["dps"] = "ff96ad", 	-- DPS
			["spd"] = "fff250",		-- Speed
			["arm"] = "b9b9b9", 	-- Armor
			
			["sp"]	= "ff73ba",		-- Spell Power
			["mr"]	= "ff73ba",		-- Mastery Rating
			
			-- Specials
			["ench"] = "00ff00",	-- Ench: Color
			
			-- Unspecified
			['*']	 = "ffffff",	-- Default for unspecified
		},
	}
}


function ShinyStats:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("ShinyStatsDB", defaults)
	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
	enchColor = self.db.profile.colors.ench;
	-- Bind patterns to colors
	stats = {
	{L["%+?%d+ Agility"],	self.db.profile.colors.agi},													-- Agility
	{L["%+?%d+ Stamina"],	self.db.profile.colors.sta},													-- Stamina
	{L["%+?%d+ Spirit"],	self.db.profile.colors.spi},													-- Spirit
	{L["%+?%d+ Intellect"],	self.db.profile.colors.int},													-- Intellect
	{L["%+?%d+ Strength"],	self.db.profile.colors.str},													-- Strength
	
	{L["%+?%d+ Attack Power"],	self.db.profile.colors.ap},													-- Attack Power
	{L["Equip: Increases attack power by %d+%.*"], self.db.profile.colors.ap},								-- Attack Power
	{L["attack power by %d+%.*"], self.db.profile.colors.ap},												-- Attack Power
				
					
	{L["Equip: Increases your expertise rating by %d+%.*"],	self.db.profile.colors.exp},					-- Expertise
	{L["expertise rating by %d+%.*"],	self.db.profile.colors.exp},										-- Expertise
	{L["%+?%d+ Expertise Rating"],	self.db.profile.colors.exp},											-- Expertise					
							
	{L["Equip: Increases your haste rating by %d+%.*"],	self.db.profile.colors.has},						-- Haste
	{L["haste rating by %d+%.*"],	self.db.profile.colors.has},											-- Haste
	{L["%+?%d+ Haste Rating"],	self.db.profile.colors.has},												-- Haste
					
	{L["Equip: Increases your hit rating by %d+%.*"],	self.db.profile.colors.hit},						-- Hit	
	{L["hit rating by %d+%.*"],	self.db.profile.colors.hit},												-- Hit
	{L["%+?%d+ Hit Rating"],	self.db.profile.colors.hit},												-- Hit						
					
	{L["Equip: Increases your critical strike rating by %d+%.*"],	self.db.profile.colors.cri},			-- Crit	
	{L["critical strike rating by %d+%.*"],	self.db.profile.colors.cri},									-- Crit	
	{L["%+?%d+ Critical Strike Rating"],	self.db.profile.colors.cri},									-- Crit
	{L["%+?%d+ Crit Rating"],	self.db.profile.colors.cri},												-- Crit	
					
	{L["Item Level %d*"],	self.db.profile.colors.ilvl},													-- Item Level
					
	{L["%+?%d+ Armor"],	self.db.profile.colors.arm},														-- Armor

	{L["Equip: Increases your parry rating by %d*%.*"], self.db.profile.colors.par},						-- Parry
	{L["parry rating by %d*%.*"], self.db.profile.colors.par},												-- Parry
	{L["%+?%d+ Parry"],	self.db.profile.colors.par},														-- Parry

	{L["Equip: Increases your block rating by %d*%.*"], self.db.profile.colors.blo},						-- Block
	{L["block rating by %d*%.*"], self.db.profile.colors.blo},												-- Block	
	{L["%+?%d+ Block"],	self.db.profile.colors.blo},														-- Block	

	{L["Equip: Increases your dodge rating by %d*%.*"],	self.db.profile.colors.dod},						-- Dodge
	{L["dodge rating by %d*%.*"], self.db.profile.colors.dod},												-- Dodge		
	{L["%+?%d+ Dodge Rating"],	self.db.profile.colors.dod},												-- Dodge
					
	{L["Equip: Increases your resilience rating by %d*%.*"],	self.db.profile.colors.res},				-- Resilience
	{L["resilience rating by %d*%.*"], self.db.profile.colors.res},											-- Resilience	
	{L["%+?%d+ Resilience Rating"],	self.db.profile.colors.res},											-- Resilience

	{L["%+?%d+ Fire Resistance"],	self.db.profile.colors.fr},												-- Fire Res
	{L["%+?%d+ Frost Resistance"],	self.db.profile.colors.frr},											-- Frost Res
	{L["%+?%d+ Nature Resistance"],	self.db.profile.colors.nr},												-- Nature Res
	{L["%+?%d+ Shadow Resistance"],	self.db.profile.colors.sr},												-- Shadow Res
	{L["%+?%d+ Holy Resistance"],	self.db.profile.colors.hr},												-- Holy Res
	{L["%+?%d+ Arcane Resistance"],	self.db.profile.colors.ar},												-- Arcane Res
					
	{L["Equip: Increases spell power by %d*%.*"],	self.db.profile.colors.sp},								-- Spell Power
	{L["spell power by %d*%.*"],	self.db.profile.colors.sp},												-- Spell Power
	{L["%+?%d+ Spell Power"],	self.db.profile.colors.sp},													-- Spell Power
					
	{L["%(%d+%.*%d* damage per second%)"],	self.db.profile.colors.dps},									-- DPS
	
	{L["Speed %d+%.*%d*"],	self.db.profile.colors.spd},													-- Speed

	{L["Equip: Increases your mastery rating by %d*%.*"],	self.db.profile.colors.mr},						-- Mastery Rating
	{L["mastery rating by %d*%.*"],	self.db.profile.colors.mr},												-- Mastery Rating
	{L["%+?%d+ Mastery Rating"],	self.db.profile.colors.mr},												-- Mastery Rating
	
	{L["Ench"], self.db.profile.colors.ench},																-- Ench: Color
	
}

	shortenEnchants, shortenEquips = self.db.profile.shortenEnchants, self.db.profile.shortenEquips
	
end

function ShinyStats:RefreshConfig()
  -- would do some stuff here
end

local function ExtendTooltip(tip, itemLink)
		local regions = {tip:GetRegions()}
		for i = 1, #regions do
			local region = regions[i]
			if region and region:GetObjectType() == "FontString" then
				local r,g = region:GetTextColor()
				local text = region:GetText()
				if text then
					local newText,m,mtext = nil, nil, nil
					
					-- Check if Socket bonus is activated (also check if you have reached the end part where no modification will be needed, then "continue")
					if (string.match(text,L["^Socket Bonus"]) and r < 0.51 and g < 0.51) or string.match(text,L["^<Shift"]) or string.match(text,L["^<Right"]) or string.match(text,L["^Set"]) then
						-- Continue
						-- Do nothing, stupid LUA lacks continue statements and a break kills the main loop
					else
						-- Add "Ench: " to enchants / Munch and's
						if not string.match(text,L["^Socket"]) and not string.match(text,L["^Equip"]) and not string.match(text,L["^Chance"]) and not string.match(text,L["^Set"]) and not string.match(text,L["^Use"]) then
							
							-- Add "Ench: " to enchants
							if r < 0.05 and g > 0.95 then
								text = format("|cff%s%s: %s|r",enchColor,L["Ench"],text)
							end
							
							-- Munch 'ands' and replace with single space
							if shortenEnchants then
								text = gsub(text,L[" and "]," ")
							end
							if shortenEquips then
								-- Do something later on in future
							end
						end
						
						for i,v in ipairs(stats) do 
							-- Check if newText is already set (meaning there might be more matches on a single line)
							if newText ~= nil then
								mtext = newText
							else
								mtext = text
							end
							-- Perform matching against stats table
							newText,m = gsub(mtext,v[1],("|cff%s%s|r"):format(v[2],"%1"))
							
							if m > 0 then -- If any match found, set the text
								region:SetText(newText)
							end
						end
					end
				end
			end
		end
		tip:Show()
end

local function hookTip(tooltip, method, action)
  if not tooltip then return end
  hooksecurefunc(tooltip, method, function(tip, ...)
    local link, count = action(...)
    if link then
      ExtendTooltip(tip, link)
    end
  end)
end

local function hookCompareTip(tooltip)
  if not tooltip then return end
  hooksecurefunc(tooltip, 'SetHyperlinkCompareItem', function(tip, mainLink)
    local _, link = tip:GetItem()
    if link then
      ExtendTooltip(tip, link)
    end
  end)
end

local function hookTipScript(tooltip)
  if tooltip and tooltip.HookScript then
    tooltip:HookScript('OnTooltipSetItem', function(tip, ...)
      local _, link = tip:GetItem()
      if link and GetItemInfo(link) then
        ExtendTooltip(tip, link)
      end
    end)
  end
end

local hooked = false
local function HookTooltips()
  if hooked then return end
  hooked = true

  hookTipScript(GameTooltip)
  hookTipScript(ItemRefTooltip)

  hookCompareTip(ShoppingTooltip1)
  hookCompareTip(ShoppingTooltip2)
  hookCompareTip(ShoppingTooltip3)
  hookCompareTip(ItemRefShoppingTooltip1)
  hookCompareTip(ItemRefShoppingTooltip2)
  hookCompareTip(ItemRefShoppingTooltip3)

  if LinkWrangler and LinkWrangler.RegisterCallback then
    LinkWrangler.RegisterCallback("ShinyStats", function(frame,link)
      ExtendTooltip(frame,link)
    end, "item")
  end
end

-- The following stuff is unfortunately necessary, because some addons *override*
-- the tooltip handlers instead of hooking them :(

local f = CreateFrame("Frame")
local entered = false
local elapsed = 0
local timerStarted = false
f:SetScript("OnEvent", function(f, e, ...) f[e](f, ...) end)

function f:PLAYER_ENTERING_WORLD()
	entered = true
	f:ScheduleHook()
end

function f:ADDON_LOADED()
	if entered then
		f:ScheduleHook()
	end
end

function f:ScheduleHook()
	if timerStarted then
		elapsed = 0
	else
		f:SetScript("OnUpdate", f.OnTimer)
	end
end

function f:OnTimer(eps)
	elapsed = elapsed + eps
	if elapsed >= 3 then
		f:SetScript("OnUpdate", nil)
		HookTooltips()
	end
end

f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("ADDON_LOADED")

